from hotsos.core.log import log
from hotsos.core.plugins.kernel.kernlog.common import KernLogBase
from hotsos.core.search import SearchDef


class OverMTUDroppedPacketEvent(object):

    @property
    def searchdef(self):
        return SearchDef(r'.+\] (\S+): dropped over-mtu packet',
                         hint='dropped', tag='over-mtu-dropped')


class KernLogEvents(KernLogBase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for event in [OverMTUDroppedPacketEvent()]:
            self.searcher.add(event.searchdef, self.path)

        self.results = self.searcher.run()

    @property
    def over_mtu_dropped_packets(self):
        """
        Return a tally of interfaces that have reports of over-mtu dropped
        packets in kern.log.

        Interfaces are only included in the result if they meet the following
        requirements:

          1. they are not an OpenvSwitch bridge
          2. they exist on the localhost and are not namespaced

        @return: dict of interfaces and an integer count of associated dropped
                 packet messages.
        """
        interfaces = {}
        for r in self.results.find_by_tag('over-mtu-dropped'):
            if r.get(1) in interfaces:
                interfaces[r.get(1)] += 1
            else:
                interfaces[r.get(1)] = 1

        if interfaces:
            # only report on interfaces that currently exist
            host_interfaces = [iface.name for iface in
                               self.hostnet_helper.host_interfaces_all]
            # filter out interfaces that are actually ovs bridge aliases
            ovs_bridges = self.cli_helper.ovs_vsctl_list_br()
            # strip trailing newline chars
            ovs_bridges = [br.strip() for br in ovs_bridges]

            interfaces_extant = {}
            for name, drops in interfaces.items():
                if name in host_interfaces:
                    if name not in ovs_bridges:
                        interfaces_extant[name] = drops
                    else:
                        log.debug("excluding ovs bridge %s", name)

            if interfaces_extant:
                # sort by number of occurrences
                sorted_dict = {}
                for k, v in sorted(interfaces_extant.items(),
                                   key=lambda e: e[1], reverse=True):
                    sorted_dict[k] = v

                return sorted_dict

        return {}



from hotsos.core.log import log
from hotsos.core.ycheck.engine.properties.requires import (
    intercept_exception,
    YRequirementTypeWithOpsBase,
)


class YPropertyVarOps(YRequirementTypeWithOpsBase):
    _override_keys = ['varops']
    _overrride_autoregister = True

    @property
    def input(self):
        return self.content[0][0]

    @property
    def ops(self):
        return self.content[1:]

    def _apply_ops(self):
        actual = self.resolve_var(self.input)
        result = self.apply_ops(self.ops, opinput=actual)
        log.debug('requirement check: varref %s %s (result=%s)',
                  actual, self.ops_to_str(self.ops), result)
        self.cache.set('ops', self.ops_to_str(self.ops))
        self.cache.set('input_value', actual)
        self.cache.set('input_ref', self.input)
        return result

    @property
    @intercept_exception
    def _result(self):
        return self._apply_ops()



import os

from searchkit import (   # noqa: F403,F401, pylint: disable=W0611
    FileSearcher as _FileSearcher,
    ResultFieldInfo,
    SearchDef,
    SequenceSearchDef,
)
from searchkit.constraints import (
    SearchConstraintSearchSince as _SearchConstraintSearchSince
)
from hotsos.core.config import HotSOSConfig
from hotsos.core.host_helpers.cli import CLIHelper
from hotsos.core.log import log


class FileSearcher(_FileSearcher):
    def __init__(self, *args, **kwargs):
        if HotSOSConfig.use_all_logs:
            max_logrotate_depth = HotSOSConfig.max_logrotate_depth
        else:
            max_logrotate_depth = 1

        super().__init__(*args,
                         max_parallel_tasks=HotSOSConfig.max_parallel_tasks,
                         max_logrotate_depth=max_logrotate_depth,
                         **kwargs)


class SearchConstraintSearchSince(_SearchConstraintSearchSince):
    def __init__(self, *args, **kwargs):
        if 'days' not in kwargs and 'hours' not in kwargs:
            days = 1
            if HotSOSConfig.use_all_logs:
                days = HotSOSConfig.max_logrotate_depth

            kwargs['days'] = days

        current_date = CLIHelper().date(format='+%Y-%m-%d %H:%M:%S')
        if not current_date or not isinstance(current_date, str):
            log.error("current date '%s' being provided to search "
                      "constraints is not valid.", current_date)
            return

        super().__init__(*args, current_date=current_date, **kwargs)

    def apply_to_line(self, *args, **kwargs):
        if not os.path.isdir(os.path.join(HotSOSConfig.data_root,
                                          'sos_commands')):
            log.info("skipping line constraint since data_root is not a "
                     "sosreport therefore files may be changing")
            return True

        return super().apply_to_line(*args, **kwargs)

    def apply_to_file(self, *args, **kwargs):
        if not os.path.isdir(os.path.join(HotSOSConfig.data_root,
                                          'sos_commands')):
            log.info("skipping file constraint since data_root is not a "
                     "sosreport therefore files may be changing")
            return 0

        return super().apply_to_file(*args, **kwargs)



from hotsos.core.plugins.vault import VaultChecksBase


class VaultSummary(VaultChecksBase):
    summary_part_index = 0

    def __0_summary_services(self):
        if self.systemd.services:
            return self.systemd.summary
        if self.pebble.services:
            return self.pebble.summary

    def __1_summary_snaps(self):
        if self.snaps.core:
            return self.snaps.all_formatted
